import base64
import hmac
import json
import requests
import urllib
import urllib.request
from urllib.parse import urljoin
from time import time
import ccxt
from ccxt.base.decimal_to_precision import decimal_to_precision, TRUNCATE
from ccxtools.base.CcxtFutureExchange import CcxtFutureExchange

from ccxtools.tools import add_query_to_url


class Bingx(CcxtFutureExchange):
    BASE_URL = 'https://api-swap-rest.bingx.com/api/v1'

    def __init__(self, who, env_vars):
        super().__init__(env_vars)
        self.ccxt_inst = ccxt.bingx({
            'apiKey': env_vars(f'BINGX_API_KEY{who}'),
            'secret': env_vars(f'BINGX_SECRET_KEY{who}'),
            'options': {
                'defaultType': 'swap'
            }
        })
        self.contract_sizes = self.get_contract_sizes()

        self.API_KEY = env_vars(f'BINGX_API_KEY{who}')
        self.SECRET_KEY = env_vars(f'BINGX_SECRET_KEY{who}')

    def get_last_price(self, ticker: str) -> float:
        res = self.ccxt_inst.fetch_ticker(f'{ticker}-USDT')
        return res['last']

    def get_funding_rates(self):
        data_list = self.ccxt_inst.swap_v2_public_get_quote_premiumindex()['data']

        if len(data_list) == 0:
            raise Exception('No funding rate data')

        funding_rates = {}
        for rate_data in data_list:
            ticker = rate_data['symbol'][:rate_data['symbol'].find('-')]
            funding_rates[ticker] = float(rate_data['lastFundingRate'])

        return funding_rates

    def get_contract_sizes(self):
        """
        :return: {
            'BTC': 0.1,
            'ETH': 0.01,
            ...
        }
        """
        contracts = self.ccxt_inst.fetch_markets()

        sizes = {}
        for contract in contracts:
            if not contract['linear']:
                continue

            ticker = contract['base']
            size = contract['contractSize']

            sizes[ticker] = size

        return sizes

    def get_balance(self, ticker):
        """
        :param ticker: <String> ticker name. ex) 'USDT', 'BTC'
        :return: <Int> balance amount
        """
        res = self.ccxt_inst.fetch_balance()
        if ticker in res['total']:
            return res['total'][ticker]
        return 0

    def get_position(self, ticker: str) -> float:
        total = 0

        positions = self.ccxt_inst.swap_v2_private_get_user_positions({
            'symbol': f'{ticker}-USDT',
        })['data']

        for position in positions:
            absolute_amount = float(position['positionAmt'])
            if position['positionSide'] == 'LONG':
                total += absolute_amount
            else:
                total -= absolute_amount
        return total

    def post_market_order(self, ticker, side, open_close, amount):
        """
        :param ticker: <String>
        :param side: <Enum: "buy" | "sell">
        :param open_close: <Enum: "open" | "close">
        :param amount: <Float | Int>
        :return: <Float> average filled price
        """
        symbol = f'{ticker}-USDT'
        res = self.ccxt_inst.create_market_order(symbol, side, amount, params={
            'reduceOnly': open_close == 'close',
        })

        try:
            order_id = res['info']['orderId']
        except:
            raise Exception(res)
        order_info = self.ccxt_inst.fetch_order(order_id, symbol)
        return order_info['average']
